@@ -47,90 +47,114 @@ from pykson import JsonObject, Pykson, IntegerField, StringField, ObjectListFiel
 # from pykson import Pykson, JsonObject, IntegerField, StringField, ObjectField
 
 
-class Score(JsonObject):
-    score = IntegerField(serialized_name='s')
-    course = StringField(serialized_name='c')
-    data = JsonField(serialized_name='d')
+# class Score(JsonObject):
+#     score = IntegerField(serialized_name='s')
+#     course = StringField(serialized_name='c')
+#     data = JsonField(serialized_name='d')
+#
+#
+# class School(JsonObject):
+#     name = StringField(serialized_name='n')
+#
+#
+# class Student(JsonObject):
+#
+#     first_name = StringField(serialized_name="fn")
+#     last_name = StringField(serialized_name="ln")
+#     age = IntegerField(serialized_name="a")
+#     school = ObjectField(School, serialized_name="sc")
+#     scores = ObjectListField(Score, serialized_name="s")
+#
+#
+# # json_text = '{"fn":"John", "ln":"Smith", "a": 25, "sc": {"n": "MIT"}, "s": [{"s": 100, "c":"Algebra", "d": {"m" :"Metod"}}]}'
+# # student = Pykson().from_json(json_text, Student)
+# # print(Pykson().to_json(student))
+# # a = student.scores[0]
+# # student.school
+#
+#
+# class Course(JsonObject):
+#     name = StringField(serialized_name='n')
+#
+#
+# class MathCourse(Course):
+#     teacher_name = StringField(serialized_name='tn')
+#
+#     def __str__(self):
+#         return str(self.name) + ' ' + str(self.teacher_name)
+#
+#     def __repr__(self):
+#         return self.__str__()
+#
+#
+# class FootballCourse(Course):
+#     coach_name = StringField(serialized_name='cn')
+#
+#     def __str__(self):
+#         return str(self.name) + ' ' + str(self.coach_name)
+#
+#     def __repr__(self):
+#         return self.__str__()
+#
+#
+# class Semester(JsonObject):
+#     course1 = ObjectField(Course, serialized_name="c1")
+#     course2 = ObjectField(Course, serialized_name="c2")
+#
+#     def __str__(self):
+#         return '{c1:' + str(self.course1) + ', c2:' + str(self.course2)+'}'
+#
+#     def __repr__(self):
+#         return self.__str__()
+#
+# pson = Pykson()
+#
+# class J0(JsonObject):
+#     a = StringField(serialized_name='a')
+#
+#
+# class J1(J0):
+#     b = StringField(serialized_name='b')
+#
+#
+# class J2(J1):
+#     c = StringField(serialized_name='c')
+#
+# aa = J2(a='1', b='2', c='3')
+#
+# print(aa.a)
+#
+# jj = pson.to_json(aa)
+# print(jj)
+#
+# bb = pson.from_json('{"c": "3", "b": "2", "a": "4"}', J2)
+#
+# print(bb.a)
+# print(pson.to_json(bb))
+
+from enum import Enum
+
+from pykson import EnumIntegerField
+
+class MyEnum(Enum):
+    A = 1
+    B = 2
 
 
-class School(JsonObject):
-    name = StringField(serialized_name='n')
+class MyJson(JsonObject):
+    a = ListField(serialized_name='aaa', item_type=EnumIntegerField(null=False, enum=MyEnum))
+    b = ListField(serialized_name='bbb', item_type=str)
 
 
-class Student(JsonObject):
-
-    first_name = StringField(serialized_name="fn")
-    last_name = StringField(serialized_name="ln")
-    age = IntegerField(serialized_name="a")
-    school = ObjectField(School, serialized_name="sc")
-    scores = ObjectListField(Score, serialized_name="s")
-
-
-# json_text = '{"fn":"John", "ln":"Smith", "a": 25, "sc": {"n": "MIT"}, "s": [{"s": 100, "c":"Algebra", "d": {"m" :"Metod"}}]}'
-# student = Pykson().from_json(json_text, Student)
-# print(Pykson().to_json(student))
-# a = student.scores[0]
-# student.school
-
-
-class Course(JsonObject):
-    name = StringField(serialized_name='n')
-
-
-class MathCourse(Course):
-    teacher_name = StringField(serialized_name='tn')
-
-    def __str__(self):
-        return str(self.name) + ' ' + str(self.teacher_name)
-
-    def __repr__(self):
-        return self.__str__()
-
-
-class FootballCourse(Course):
-    coach_name = StringField(serialized_name='cn')
-
-    def __str__(self):
-        return str(self.name) + ' ' + str(self.coach_name)
-
-    def __repr__(self):
-        return self.__str__()
-
-
-class Semester(JsonObject):
-    course1 = ObjectField(Course, serialized_name="c1")
-    course2 = ObjectField(Course, serialized_name="c2")
-
-    def __str__(self):
-        return '{c1:' + str(self.course1) + ', c2:' + str(self.course2)+'}'
-
-    def __repr__(self):
-        return self.__str__()
+mj = MyJson(a=[MyEnum.A, MyEnum.B], b=['1', '3'])
 
 pson = Pykson()
+mjj = pson.to_json(mj)
 
-class J0(JsonObject):
-    a = StringField(serialized_name='a')
+print(mjj)
 
+print(pson.to_json(pson.from_json(mjj, MyJson)))
 
-class J1(J0):
-    b = StringField(serialized_name='b')
-
-
-class J2(J1):
-    c = StringField(serialized_name='c')
-
-aa = J2(a='1', b='2', c='3')
-
-print(aa.a)
-
-jj = pson.to_json(aa)
-print(jj)
-
-bb = pson.from_json('{"c": "3", "b": "2", "a": "4"}', J2)
-
-print(bb.a)
-print(pson.to_json(bb))
 
 # pson = Pykson()
 # pson.register_type_hierarchy_adapter(TypeHierarchyAdapter(
